from collections import OrderedDict
import re
from elifecleaner.utils import pad_msid

JOURNAL = "elife"

STRING_TO_TERM_MAP = {
    "animation": "video",
    "video": "video",
    "appendix": "app",
    "figure": "fig",
    "video supplement": "video",
    "videos supplement": "video",
    "supplementary video": "video",
}

IGNORE_TERM = "audio"

# todo!!!
# - raise exception when ignore term is found, or when video term is not found, to distinguish between them
# - enhance regex to handle mixed numeric values like 6A


def video_file_list(files):
    'get a list of file tags from the XML with @file-type="video"'
    return [
        file_data
        for file_data in files
        if isinstance(file_data, dict) and file_data.get("file_type") == "video"
    ]


def collect_video_data(files):
    "from the list of files collect file name and metadata for videos"
    video_files = video_file_list(files)
    video_data = []
    for file_data in video_files:
        data = OrderedDict()
        data["upload_file_nm"] = file_data.get("upload_file_nm")
        for meta in file_data.get("custom_meta"):
            if meta.get("meta_name") == "Title":
                data["title"] = meta.get("meta_value")
                break
        if len(data.keys()) > 1:
            video_data.append(data)
    return video_data


def rename_video_data(video_data, article_id):
    "generate new video filename and id from the video data"
    generated_video_data = []
    for data in video_data:
        video_data_output = OrderedDict()
        video_data_output["upload_file_nm"] = data.get("upload_file_nm")
        video_data_output["video_id"] = video_id(data.get("title"))
        video_data_output["video_filename"] = video_filename(
            data.get("title"), data.get("upload_file_nm"), article_id
        )
        generated_video_data.append(video_data_output)
    return generated_video_data


def video_data_from_files(files, article_id):
    "from a list of files return video data to be used in renaming files and XML"
    video_data = collect_video_data(files)
    return rename_video_data(video_data, article_id)


def all_terms(titles):
    "get a list of all terms for the title values"
    term_map = OrderedDict()
    for title in titles:
        term_map[title] = terms_from_title(title)
    # todo!! check all title values are non-None

    # check for duplicates
    a_list = [str(term) for term in term_map.values()]
    unique_terms = list({str(term) for term in term_map.values()})
    if len(a_list) != len(unique_terms):
        # todo!! handle duplicates
        return None
    return term_map


def terms_from_title(title):
    "from a title string extract video terms and numbers"
    terms = []
    # ignore the value if audio is in the title
    if IGNORE_TERM in title.lower():
        return []
    # convert some punctuation to space for more lenient matching
    for char in ["_", "-"]:
        title = title.replace(char, " ")
    match_pattern = re.compile(r"(\D*?)(\d+)")
    for match in match_pattern.findall(title):
        section_term = match[0].lstrip(" -").strip().lower()
        if section_term in STRING_TO_TERM_MAP:
            term = OrderedDict()
            term["name"] = STRING_TO_TERM_MAP.get(section_term)
            term["number"] = match[1]
            terms.append(term)
    # check video is one of the name values
    if "video" in [term.get("name", "") for term in terms]:
        return terms
    return []


def video_id(title):
    "generate an id attribute for a video from its title string"
    id_string = ""
    terms = terms_from_title(title)
    if not terms:
        return None
    for term in terms:
        id_string += "%s%s" % (term.get("name"), term.get("number"))
    return id_string


def video_filename(title, upload_file_nm, article_id, journal=JOURNAL):
    "generate a new file name for a video file"
    terms = terms_from_title(title)
    if not terms:
        return None
    new_filename_parts = []
    new_filename_parts.append(journal)
    new_filename_parts.append(pad_msid(article_id))
    for term in terms:
        new_filename_parts.append("%s%s" % (term.get("name"), term.get("number")))
    new_filename = "-".join(new_filename_parts)
    # add file extension
    file_extension = upload_file_nm.split(".")[-1]
    new_filename = "%s.%s" % (new_filename, file_extension)
    return new_filename



import os
import zipfile
from collections import OrderedDict


def profile_zip(file_name):
    """open the zip and get relevant file names"""
    zip_asset_infos = []
    with zipfile.ZipFile(file_name, "r") as open_zipfile:
        for zipfile_info in open_zipfile.infolist():
            zip_asset_infos.append(zipfile_info)
    # sort by file name
    zip_asset_infos = sorted(zip_asset_infos, key=lambda asset: asset.filename)
    return zip_asset_infos


def unzip_file(open_zipfile, zip_file_info, output_path):
    "read the zip_file_info from the open_zipfile and write to output_path"
    with open(output_path, "wb") as output_file:
        output_file.write(open_zipfile.read(zip_file_info))


def unzip_zip(file_name, temp_dir):
    "unzip certain files and return the local paths"
    asset_file_name_map = OrderedDict()
    zip_asset_infos = profile_zip(file_name)

    # extract the files
    with zipfile.ZipFile(file_name, "r") as open_zipfile:
        for zip_asset_info in zip_asset_infos:
            file_name_parts = zip_asset_info.filename.split("/")
            file_name = file_name_parts[-1]
            folder_name = ""
            if len(file_name_parts) > 1:
                folder_name = file_name_parts[-2]
                folder_path = os.path.join(temp_dir, folder_name)
                if not os.path.exists(folder_path):
                    os.mkdir(folder_path)
            asset_file_name = os.path.join(temp_dir, folder_name, file_name)
            unzip_file(open_zipfile, zip_asset_info, asset_file_name)
            asset_file_name_map[zip_asset_info.filename] = asset_file_name

    return asset_file_name_map



import time
from xml.dom import minidom
from xml.etree import ElementTree
from xml.etree.ElementTree import Element, SubElement
from elifearticle import parse as articleparse
from elifearticle.article import ArticleDate
from elifecleaner import parse
from elifecleaner.utils import pad_msid

JOURNAL_ID_TYPES = ["nlm-ta", "publisher-id"]

NAMESPACE_MAP = {
    "xmlns:mml": "http://www.w3.org/1998/Math/MathML",
    "xmlns:xlink": "http://www.w3.org/1999/xlink",
}

MEDIA_CONTENT_TYPE = "glencoe play-in-place height-250 width-310"

DTD_VERSION = "1.1d1"


def article_from_xml(xml_file_path):
    return articleparse.build_article_from_xml(xml_file_path)


def set_pub_date(parent, article, pub_type):
    "set XML pub-date tag to values from an article date"
    pub_date = article.get_date(pub_type)
    if pub_date:
        date_tag = SubElement(parent, "pub-date")
        date_tag.set("date-type", pub_type)
        if pub_date.publication_format:
            date_tag.set("publication-format", pub_date.publication_format)
        day = SubElement(date_tag, "day")
        day.text = str(pub_date.date.tm_mday).zfill(2)
        month = SubElement(date_tag, "month")
        month.text = str(pub_date.date.tm_mon).zfill(2)
        year = SubElement(date_tag, "year")
        year.text = str(pub_date.date.tm_year)


def set_article_meta(parent, article):
    "set XML article-meta tag and tags found inside it"
    article_meta = SubElement(parent, "article-meta")

    # article-id pub-id-type="publisher-id"
    if article.manuscript:
        pub_id_type = "publisher-id"
        article_id = SubElement(article_meta, "article-id")
        article_id.text = pad_msid(article.manuscript)
        article_id.set("pub-id-type", pub_id_type)

    # article-id pub-id-type="doi"
    if article.doi:
        pub_id_type = "doi"
        article_id = SubElement(article_meta, "article-id")
        article_id.text = article.doi
        article_id.set("pub-id-type", pub_id_type)

    set_pub_date(article_meta, article, "publication")


def set_front(parent, journal_data, article):
    "set XML front tag and the tags found inside it"
    front_tag = SubElement(parent, "front")
    journal_meta = SubElement(front_tag, "journal-meta")

    # journal-id
    if journal_data.get("journal_ids"):
        for journal_id_type, value in journal_data.get("journal_ids").items():
            # concatenate the config name to look for the value
            if journal_id_type:
                journal_id = SubElement(journal_meta, "journal-id")
                journal_id.set("journal-id-type", journal_id_type)
                journal_id.text = value
    # journal-title-group
    journal_title_group = SubElement(journal_meta, "journal-title-group")

    # journal-title
    if journal_data.get("journal_title"):
        journal_title_tag = SubElement(journal_title_group, "journal-title")
        journal_title_tag.text = journal_data.get("journal_title")

    # publisher
    if journal_data.get("publisher_name"):
        publisher = SubElement(journal_meta, "publisher")
        publisher_name_tag = SubElement(publisher, "publisher-name")
        publisher_name_tag.text = journal_data.get("publisher_name")

    set_article_meta(front_tag, article)


def set_body(parent, video_data):
    "set XML body tag contents"
    body_tag = SubElement(parent, "body")
    # video media tags
    for video in video_data:
        media_tag = SubElement(body_tag, "media")
        media_tag.set("xlink:href", video.get("video_filename"))
        media_tag.set("id", video.get("video_id"))
        media_tag.set("content-type", MEDIA_CONTENT_TYPE)
        media_tag.set("mimetype", "video")


def output_xml(root, pretty=False, indent=""):
    "output root XML Element to a string"
    encoding = "utf-8"
    rough_string = ElementTree.tostring(root, encoding)
    reparsed = minidom.parseString(rough_string)

    if pretty is True:
        return reparsed.toprettyxml(indent, encoding=encoding)
    return reparsed.toxml(encoding=encoding)


def generate_xml(article, journal_data, video_data, pretty=True, indent=""):
    "from jats_content generate final JATS output"

    # set the date to today
    article_date = ArticleDate("publication", time.gmtime())
    article_date.publication_format = "electronic"
    article.add_date(article_date)

    # Create the root XML node
    root = Element("article")
    # set attributes
    root.set("dtd-version", DTD_VERSION)
    root.set("article-type", article.article_type)
    # set namespaces
    for prefix, url in NAMESPACE_MAP.items():
        root.set(prefix, url)

    set_front(root, journal_data, article)
    set_body(root, video_data)

    return output_xml(root, pretty, indent)


def glencoe_xml(xml_file_path, video_data, pretty=True, indent=""):
    "generate XML to be submitted to Glencoe"
    # build an Article object from the XML
    article, error_count = article_from_xml(xml_file_path)
    # collect journal data from the XML elementtree
    root = parse.parse_article_xml(xml_file_path)
    journal_ids = parse.xml_journal_id_values(root)
    filtered_journal_ids = {
        key: value for key, value in journal_ids.items() if key in JOURNAL_ID_TYPES
    }
    journal_title = parse.xml_journal_title(root)
    publisher_name = parse.xml_publisher_name(root)
    journal_data = {
        "journal_ids": filtered_journal_ids,
        "journal_title": journal_title,
        "publisher_name": publisher_name,
    }
    return generate_xml(article, journal_data, video_data, pretty, indent)
